﻿using FluentValidation.AspNetCore;
using Hangfire;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Http.Internal;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Performance.DtoModels;
using Performance.EntityModels;
using Performance.Infrastructure;
using Performance.Services;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Linq.Expressions;
using System.Threading.Tasks;

// For more information on enabling Web API for empty projects, visit https://go.microsoft.com/fwlink/?LinkID=397860

namespace Performance.Api.Controllers
{
    /// <summary>
    /// 科室二次分配
    /// </summary>
    [Route("api/[controller]")]
    public class AgainAllotController : Controller
    {
        private AgainAllotService againAllotService;
        private ClaimService claimService;
        private AllotService allotService;
        private IHostingEnvironment evn;
        private ConfigService configService;
        public AgainAllotController(AgainAllotService againAllotService,
             ClaimService claimService,
             AllotService allotService,
             IHostingEnvironment evn,
             ConfigService configService)
        {
            this.againAllotService = againAllotService;
            this.claimService = claimService;
            this.allotService = allotService;
            this.evn = evn;
            this.configService = configService;
        }

        /// <summary>
        /// 返回当前用户医院下绩效列表
        /// </summary>
        /// <returns></returns>
        [Route("allotlist")]
        [HttpPost]
        public ApiResponse AllotList([FromBody]UserRequest request)
        {
            var list = againAllotService.GetAllotList(request.ID);
            return new ApiResponse(ResponseType.OK, list);
        }

        /// <summary>
        /// 上传文件
        /// </summary>
        /// <param name="form"></param>
        /// <returns></returns>
        [Route("import")]
        [HttpPost]
        public ApiResponse Import([FromForm] IFormCollection form)
        {
            var allotid = form.ToDictionary().GetValue("allotid", 0);
            if (allotid <= 0)
                return new ApiResponse(ResponseType.Fail, "参数错误", "allotid无效");

            var userid = form.ToDictionary().GetValue("userid", 0);
            if (userid <= 0)
                return new ApiResponse(ResponseType.Fail, "参数错误", "userid无效");

            var file = ((FormFileCollection)form.Files).FirstOrDefault();
            if (file == null)
                return new ApiResponse(ResponseType.Fail, "参数错误", "文件无效");

            var allot = allotService.GetAllot(allotid);
            if (allot == null)
                return new ApiResponse(ResponseType.Fail, "allotid不存在");

            var name = FileHelper.GetFileNameNoExtension(file.FileName) + DateTime.Now.ToString("yyyyMMddHHmmssfff");
            var ext = FileHelper.GetExtension(file.FileName);
            var dpath = Path.Combine(evn.ContentRootPath, "Files", $"{allot.HospitalId}", $"{allot.Year}{allot.Month.ToString().PadLeft(2, '0')}");
            FileHelper.CreateDirectory(dpath);

            var path = Path.Combine(dpath, $"{name}{ext}");

            using (var stream = file.OpenReadStream())
            {
                byte[] bytes = new byte[stream.Length];
                stream.Read(bytes, 0, bytes.Length);
                if (!FileHelper.CreateFile(path, bytes))
                    return new ApiResponse(ResponseType.Fail, $"{file.FileName}上传失败");
                allot.Path = path;
                allot.Remark = EnumHelper.GetDescription(AllotStates.FileUploaded);
                int flag = againAllotService.Insert(allot, userid);
                if (flag == 0)
                    return new ApiResponse(ResponseType.Fail, $"{file.FileName}上传成功，修改状态失败");
                configService.ClearAgain(flag);
                configService.CopyAgain(flag);
            }

            return new ApiResponse(ResponseType.OK);
        }

        [Route("generate")]
        [HttpPost]
        public ApiResponse Generate([CustomizeValidator(RuleSet = "Generate"), FromBody]AgainAllotRequest request)
        {
            var user = claimService.At(request);
            var result = againAllotService.Generate(request, user);
            return new ApiResponse(ResponseType.OK, new { result.AgainSituation, result.SheetExport });
        }
    }
}
